<?php

/**
 * @todo decouple from this module as this is hard coupled with this project.
 * @todo Instead provide a custom field with formatter.
 */

namespace Drupal\carerix_form_ds\Plugin\DsField\Node;

use Drupal\Core\Form\FormStateInterface;
use Drupal\ds\Plugin\DsField\DsFieldBase;
use Drupal\carerix_form\Form\CRForm;

/**
 * Plugin that renders the form for submitting data into the Carerix system.
 *
 * @DsField(
 *   id = "carerix_form",
 *   title = @Translation("Carerix Form"),
 *   entity_type = "node",
 *   provider = "carerix_form",
 * )
 */
class NodeCarerixForm extends DsFieldBase {

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return array(
      'carerix_form_id' => 'open_application',
    );
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm($form, FormStateInterface $form_state) {
    // Get Carerix form config entities.
    $configEntityIds = array_values($this->getEntities());
    $em = \Drupal::entityTypeManager()->getStorage('carerix_form');
    $configEntities = $em->loadMultiple($configEntityIds);
    // Options.
    $options = [];
    foreach ($configEntities as $id => $configEntity) {
      $options[$id] = $configEntity->label();
    }
    // Get settings.
    $config = $this->getConfiguration();
    $form['carerix_form_id'] = array(
      '#type' => 'select',
      '#title' => $this->t('Carerix Form ID'),
      '#default_value' => $config['carerix_form_id'],
      '#options' => $options,
    );
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Vars.
    $config = $this->getConfiguration();
    $entity = $this->entity();
    $pubId = NULL;

    // @todo Decouple. (atm. hard coupled with "field_carerix_id")
    // Check entity publication id.
    if (isset($entity->field_carerix_id)) {
      $carerixId = $entity->field_carerix_id->getValue();
      if (isset($carerixId[0]['value']) && !empty($carerixId[0]['value'])) {
        // Get Carerix publication id from entity.
        $pubId = $carerixId[0]['value'];
      }
    }

    // Check Carerix form config.
    $carerixFormId = $config['carerix_form_id'] ?: NULL;

    // Render carerix form.
    return $this->formBuilder()->getForm(CRForm::class, $carerixFormId, $pubId);
  }

  /**
   * Returns the form builder service.
   *
   * @return \Drupal\Core\Form\FormBuilderInterface
   *   Form builder.
   */
  protected function formBuilder() {
    if (!$this->formBuilder) {
      $this->formBuilder = $this->container()->get('form_builder');
    }
    return $this->formBuilder;
  }

  /**
   * Returns the service container.
   *
   * This method is marked private to prevent sub-classes from retrieving
   * services from the container through it. Instead,
   * \Drupal\Core\DependencyInjection\ContainerInjectionInterface should be used
   * for injecting services.
   *
   * @return \Symfony\Component\DependencyInjection\ContainerInterface
   *   The service container.
   */
  private function container() {
    return \Drupal::getContainer();
  }

  /**
   * Helper function to retrieve all Carerix form configuration entities.
   *
   * @param string $entity_type_id
   *   Entity type.
   *
   * @return array|int
   *   Array of entities.
   */
  protected function getEntities($entity_type_id = 'carerix_form') {
    return \Drupal::entityQuery($entity_type_id)
      ->condition('status', 1)
      ->execute();
  }

}
